@@ -361,7 +361,10 @@ e_cal_backend_file_get_alarm_email_address (ECalBackendSync *backend, EDataCal *
 static ECalBackendSyncStatus
 e_cal_backend_file_get_static_capabilities (ECalBackendSync *backend, EDataCal *cal, char **capabilities)
 {
-	*capabilities = g_strdup (CAL_STATIC_CAPABILITY_NO_EMAIL_ALARMS);
+	*capabilities = g_strdup (CAL_STATIC_CAPABILITY_NO_EMAIL_ALARMS ","
+				  CAL_STATIC_CAPABILITY_NO_THISANDFUTURE ","
+				  CAL_STATIC_CAPABILITY_DELEGATE_SUPPORTED ","
+				  CAL_STATIC_CAPABILITY_NO_THISANDPRIOR);
 	
 	return GNOME_Evolution_Calendar_Success;
 }
@@ -1258,6 +1261,15 @@ e_cal_backend_file_get_object_list (ECalBackendSync *backend, EDataCal *cal, con
 	return GNOME_Evolution_Calendar_Success;	
 }
 
+/* Gets the list of attachments */
+static ECalBackendSyncStatus
+e_cal_backend_file_get_attachment_list (ECalBackendSync *backend, EDataCal *cal, const char *uid, const char *rid, GSList **list) 
+{
+	 
+	/* TODO implement the function */
+	return GNOME_Evolution_Calendar_Success;
+}
+
 /* get_query handler for the file backend */
 static void
 e_cal_backend_file_start_query (ECalBackend *backend, EDataCalView *query)
@@ -1865,9 +1877,6 @@ e_cal_backend_file_modify_object (ECalBackendSync *backend, EDataCal *cal, const
 
 			old = e_cal_component_get_as_string (obj_data->full_object);
 
-			e_cal_util_remove_instances (e_cal_component_get_icalcomponent (obj_data->full_object),
-						     get_rid_icaltime (comp),
-						     mod);
 			new = e_cal_component_get_as_string (comp);
 			e_cal_backend_notify_object_modified (E_CAL_BACKEND (backend), old, new);
 
@@ -1925,10 +1934,6 @@ e_cal_backend_file_modify_object (ECalBackendSync *backend, EDataCal *cal, const
 				*old_object = e_cal_component_get_as_string (obj_data->full_object);
 		}
 
-		/* remove all affected recurrences */
-		e_cal_util_remove_instances (e_cal_component_get_icalcomponent (obj_data->full_object),
-					     icaltime_from_string (rid), mod);
-
 		rrdata.cbfile = cbfile;
 		rrdata.obj_data = obj_data;
 		rrdata.rid = rid;
@@ -2490,6 +2495,7 @@ e_cal_backend_file_class_init (ECalBackendFileClass *class)
  	sync_class->get_default_object_sync = e_cal_backend_file_get_default_object;
 	sync_class->get_object_sync = e_cal_backend_file_get_object;
 	sync_class->get_object_list_sync = e_cal_backend_file_get_object_list;
+	sync_class->get_attachment_list_sync = e_cal_backend_file_get_attachment_list;
 	sync_class->get_timezone_sync = e_cal_backend_file_get_timezone;
 	sync_class->add_timezone_sync = e_cal_backend_file_add_timezone;
 	sync_class->set_default_timezone_sync = e_cal_backend_file_set_default_timezone;
