@@ -366,6 +366,16 @@ camel_imap4_folder_new (CamelStore *store, const char *full_name, CamelException
 		imap4_folder->enable_mlist = TRUE;
 	}
 	
+	if (!g_ascii_strcasecmp (full_name, "INBOX")) {
+		if (camel_url_get_param (((CamelService *) store)->url, "filter"))
+			folder->folder_flags |= CAMEL_FOLDER_FILTER_RECENT;
+		if (camel_url_get_param (((CamelService *) store)->url, "filter_junk"))
+			folder->folder_flags |= CAMEL_FOLDER_FILTER_JUNK;
+	} else if (!camel_url_get_param (((CamelService *) store)->url, "filter_junk_inbox")) {
+		if (camel_url_get_param (((CamelService *) store)->url, "filter_junk"))
+			folder->folder_flags |= CAMEL_FOLDER_FILTER_JUNK;
+	}
+	
 	imap4_folder->search = camel_imap4_search_new (((CamelIMAP4Store *) store)->engine, imap4_folder->cachedir);
 	
 	if (((CamelOfflineStore *) store)->state == CAMEL_OFFLINE_STORE_NETWORK_AVAIL) {
@@ -475,9 +485,6 @@ imap4_sync_changes (CamelFolder *folder, GPtrArray *sync, CamelException *ex)
 	int retval = 0;
 	int i, j;
 	
-	if (folder->permanent_flags == 0)
-		return 0;
-	
 	on_set = g_ptr_array_new ();
 	off_set = g_ptr_array_new ();
 	
@@ -547,39 +554,41 @@ imap4_sync (CamelFolder *folder, gboolean expunge, CamelException *ex)
 	CAMEL_SERVICE_LOCK (folder->parent_store, connect_lock);
 	
 	/* gather a list of changes to sync to the server */
-	sync = g_ptr_array_new ();
-	max = camel_folder_summary_count (folder->summary);
-	for (i = 0; i < max; i++) {
-		iinfo = (CamelIMAP4MessageInfo *) (info = camel_folder_summary_index (folder->summary, i));
-		if (iinfo->info.flags & CAMEL_MESSAGE_FOLDER_FLAGGED) {
-			camel_imap4_flags_diff (&diff, iinfo->server_flags, iinfo->info.flags);
-			diff.changed &= folder->permanent_flags;
-			
-			/* weed out flag changes that we can't sync to the server */
-			if (!diff.changed)
+	if (folder->permanent_flags) {
+		sync = g_ptr_array_new ();
+		max = camel_folder_summary_count (folder->summary);
+		for (i = 0; i < max; i++) {
+			iinfo = (CamelIMAP4MessageInfo *) (info = camel_folder_summary_index (folder->summary, i));
+			if (iinfo->info.flags & CAMEL_MESSAGE_FOLDER_FLAGGED) {
+				camel_imap4_flags_diff (&diff, iinfo->server_flags, iinfo->info.flags);
+				diff.changed &= folder->permanent_flags;
+				
+				/* weed out flag changes that we can't sync to the server */
+				if (!diff.changed)
+					camel_message_info_free(info);
+				else
+					g_ptr_array_add (sync, info);
+			} else {
 				camel_message_info_free(info);
-			else
-				g_ptr_array_add (sync, info);
+			}
+		}
+		
+		if (sync->len > 0) {
+			retval = imap4_sync_changes (folder, sync, ex);
+			
+			for (i = 0; i < sync->len; i++)
+				camel_message_info_free(sync->pdata[i]);
+			
+			g_ptr_array_free (sync, TRUE);
+			
+			if (retval == -1)
+				goto done;
 		} else {
-			camel_message_info_free(info);
+			g_ptr_array_free (sync, TRUE);
 		}
 	}
 	
-	if (sync->len > 0) {
-		retval = imap4_sync_changes (folder, sync, ex);
-		
-		for (i = 0; i < sync->len; i++)
-			camel_message_info_free(sync->pdata[i]);
-		
-		g_ptr_array_free (sync, TRUE);
-		
-		if (retval == -1)
-			goto done;
-	} else {
-		g_ptr_array_free (sync, TRUE);
-	}
-	
-	if (expunge) {
+	if (expunge && !((CamelIMAP4Folder *) folder)->read_only) {
 		ic = camel_imap4_engine_queue (engine, folder, "EXPUNGE\r\n");
 		while ((id = camel_imap4_engine_iterate (engine)) < ic->id && id != -1)
 			;
@@ -899,6 +908,13 @@ imap4_append_message (CamelFolder *folder, CamelMimeMessage *message,
 	if (appended_uid)
 		*appended_uid = NULL;
 	
+	if (((CamelIMAP4Folder *) folder)->read_only) {
+		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
+				      _("Cannot append message to folder `%s': Folder is read-only"),
+				      folder->full_name);
+		return;
+	}
+	
 	if (offline->state == CAMEL_OFFLINE_STORE_NETWORK_UNAVAIL) {
 		camel_imap4_journal_append ((CamelIMAP4Journal *) imap4_folder->journal, message, info, appended_uid, ex);
 		return;
@@ -958,10 +974,7 @@ imap4_append_message (CamelFolder *folder, CamelMimeMessage *message,
 	
  retry:
 	
-	if (appended_uid && engine->capa & CAMEL_IMAP4_CAPABILITY_UIDPLUS)
-		ic = camel_imap4_engine_queue (engine, NULL, "UID APPEND %F%s%s %L\r\n", folder, flags, date, message);
-	else
-		ic = camel_imap4_engine_queue (engine, NULL, "APPEND %F%s%s %L\r\n", folder, flags, date, message);
+	ic = camel_imap4_engine_queue (engine, NULL, "APPEND %F%s%s %L\r\n", folder, flags, date, message);
 	
 	while ((id = camel_imap4_engine_iterate (engine)) < ic->id && id != -1)
 		;
@@ -1056,12 +1069,20 @@ imap4_transfer_messages_to (CamelFolder *src, GPtrArray *uids, CamelFolder *dest
 	int i, j, n, id, dest_namelen;
 	CamelMessageInfo *info;
 	CamelIMAP4Command *ic;
+	CamelException lex;
 	GPtrArray *infos;
 	char *set;
 	
 	if (transferred_uids)
 		*transferred_uids = NULL;
 	
+	camel_exception_init (&lex);
+	imap4_sync (src, FALSE, &lex);
+	if (camel_exception_is_set (&lex)) {
+		camel_exception_xfer (ex, &lex);
+		return;
+	}
+	
 	infos = g_ptr_array_new ();
 	for (i = 0; i < uids->len; i++) {
 		if (!(info = camel_folder_summary_uid (src->summary, uids->pdata[i])))
@@ -1109,7 +1130,11 @@ imap4_transfer_messages_to (CamelFolder *src, GPtrArray *uids, CamelFolder *dest
 	for (i = 0; i < infos->len; i += n) {
 		n = camel_imap4_get_uid_set (engine, src->summary, infos, i, 10 + dest_namelen, &set);
 		
-		ic = camel_imap4_engine_queue (engine, src, "UID COPY %s %F\r\n", set, dest);
+		if (move && (engine->capa & CAMEL_IMAP4_CAPABILITY_XGWMOVE))
+			ic = camel_imap4_engine_queue (engine, src, "UID XGWMOVE %s %F\r\n", set, dest);
+		else
+			ic = camel_imap4_engine_queue (engine, src, "UID COPY %s %F\r\n", set, dest);
+		
 		while ((id = camel_imap4_engine_iterate (engine)) < ic->id && id != -1)
 			;
 		
@@ -1152,7 +1177,7 @@ imap4_transfer_messages_to (CamelFolder *src, GPtrArray *uids, CamelFolder *dest
 		
 		camel_imap4_command_unref (ic);
 		
-		if (move) {
+		if (move && !(engine->capa & CAMEL_IMAP4_CAPABILITY_XGWMOVE)) {
 			for (j = i; j < n; j++) {
 				info = infos->pdata[j];
 				camel_folder_set_message_flags (src, camel_message_info_uid (info),
